@@ -1,3 +1,5 @@
+#![cfg_attr(any(miri, test), allow(unused_unsafe, missing_docs))]
+
 pub use std::alloc::GlobalAlloc;
 use std::{
     convert::TryFrom,
@@ -10,7 +12,7 @@ use crate::{
     traits::{AbiSafeTrait, AbiSafeUnsize, AbiSafeVTable, DynPtrSafe},
 };
 
-#[cfg(not(test))]
+#[cfg(not(any(miri, test)))]
 extern "C" {
     /// Function that allocates memory suitable for storing an object of size `size`, with at least the maximum fundamental alignment for that size.
     ///
@@ -57,7 +59,7 @@ extern "C" {
     ) -> !;
 }
 
-#[cfg(test)]
+#[cfg(any(miri, test))]
 #[no_mangle]
 pub unsafe extern "C" fn xlang_allocate(
     size: xlang_host::primitives::size_t,
@@ -75,7 +77,7 @@ pub unsafe extern "C" fn xlang_allocate(
     )
 }
 
-#[cfg(test)]
+#[cfg(any(miri, test))]
 #[no_mangle]
 pub unsafe extern "C" fn xlang_allocate_aligned(
     size: xlang_host::primitives::size_t,
@@ -89,7 +91,7 @@ pub unsafe extern "C" fn xlang_allocate_aligned(
     std::alloc::alloc(layout).cast::<_>()
 }
 
-#[cfg(test)]
+#[cfg(any(miri, test))]
 #[no_mangle]
 pub unsafe extern "C" fn xlang_deallocate(
     ptr: *mut core::ffi::c_void,
@@ -112,7 +114,7 @@ pub unsafe extern "C" fn xlang_deallocate(
     }
 }
 
-#[cfg(test)]
+#[cfg(any(miri, test))]
 #[no_mangle]
 pub unsafe extern "C" fn xlang_deallocate_aligned(
     ptr: *mut core::ffi::c_void,
@@ -127,7 +129,7 @@ pub unsafe extern "C" fn xlang_deallocate_aligned(
     std::alloc::dealloc(ptr.cast::<_>(), layout);
 }
 
-#[cfg(test)]
+#[cfg(any(miri, test))]
 #[no_mangle]
 pub extern "C" fn xlang_on_allocation_failure(
     size: xlang_host::primitives::size_t,
@@ -1098,7 +1100,9 @@ pub fn handle_alloc_error(layout: Layout) -> ! {
 
 #[cfg(all(test, miri))]
 mod test {
-    use super::{xlang_allocate, xlang_deallocate};
+    use super::{
+        xlang_allocate, xlang_allocate_aligned, xlang_deallocate, xlang_deallocate_aligned,
+    };
     #[test]
     pub fn test_alloc() {
         let p = unsafe { xlang_allocate(4) } as *mut i32;
@@ -1110,4 +1114,18 @@ mod test {
         }
         unsafe { xlang_deallocate(p as *mut _, 4) }
     }
+
+    #[test]
+    pub fn test_alloc_align() {
+        #[repr(align(256))]
+        struct Foo(u8);
+        let p = unsafe { xlang_allocate_aligned(256, 256) } as *mut Foo;
+        if p.is_null() {
+            return;
+        }
+        unsafe {
+            p.write(Foo(0));
+        }
+        unsafe { xlang_deallocate_aligned(p as *mut _, 256, 256) }
+    }
 }
