@@ -1,7 +1,8 @@
 use std::borrow::Borrow;
 use std::fmt::Debug;
 use std::hash::{BuildHasher, Hash, Hasher};
-use std::iter::FromIterator;
+use std::iter::{FromIterator, FusedIterator};
+use std::marker::PhantomData;
 use std::mem::MaybeUninit;
 use std::ops::Index;
 
@@ -114,6 +115,19 @@ impl<K, V, H: BuildHasher, A: Allocator> HashMap<K, V, H, A> {
         )
     }
 
+    /// Produces an [`Iterator`] over pairs of keys and mutable values in this [`HashMap`]
+    /// Note that they keys are not mutable, to avoid mutation that would affect the implementation of [`Eq`] and [`Hash`]
+    pub fn iter_mut(&mut self) -> IterMut<'_, K, V> {
+        let begin = self.htab.as_ptr();
+        let end = unsafe { begin.add(self.buckets) };
+        IterMut {
+            buckets: begin,
+            buckets_end: end,
+            bucket_position: 0,
+            phantom: PhantomData,
+        }
+    }
+
     /// Produces an [`Iterator`] over the values in this [`HashMap`]
     pub fn values(&self) -> Values<'_, K, V> {
         Values(self.iter())
@@ -363,6 +377,43 @@ impl<K: Eq + Hash, V, H: BuildHasher, A: Allocator> HashMap<K, V, H, A> {
 
         None
     }
+
+    /// Removes an entry by Key from this map, and returns the Key and Value
+    #[allow(clippy::cast_possible_truncation)]
+    pub fn remove<Q: ?Sized + Hash + Eq>(&mut self, key: &Q) -> Option<Pair<K, V>>
+    where
+        K: Borrow<Q>,
+    {
+        if self.buckets == 0 {
+            return None;
+        }
+
+        let mut hasher = self.hash.build_hasher();
+        key.hash(&mut hasher);
+        let hash = hasher.finish() as usize % self.buckets;
+
+        let bucket = unsafe { &mut *self.htab.as_ptr().add(hash) };
+
+        for i in 0..bucket.ecount {
+            // SAFETY:
+            // bucket.ecount never exceeds 16, by the check following this loop
+            let key_val = unsafe { &mut *bucket.entries.get_unchecked_mut(i).as_mut_ptr() };
+
+            if key_val.0.borrow().eq(key) {
+                let val = unsafe { core::ptr::read(key_val) };
+                for j in i..(bucket.ecount - 1) {
+                    unsafe {
+                        core::ptr::write(
+                            bucket.entries.as_mut_ptr().add(j),
+                            core::ptr::read(bucket.entries.as_mut_ptr().add(j + 1)),
+                        );
+                    }
+                }
+                return Some(val);
+            }
+        }
+        None
+    }
 }
 
 impl<K: Clone, V: Clone, H: BuildHasher + Clone, A: Allocator + Clone> Clone
@@ -473,6 +524,16 @@ impl<'a, K, V, H: BuildHasher, A: Allocator> IntoIterator for &'a HashMap<K, V,
     }
 }
 
+impl<'a, K, V, H: BuildHasher, A: Allocator> IntoIterator for &'a mut HashMap<K, V, H, A> {
+    type Item = Pair<&'a K, &'a mut V>;
+
+    type IntoIter = IterMut<'a, K, V>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.iter_mut()
+    }
+}
+
 impl<'a, K: Hash, V: Hash, H: BuildHasher, A: Allocator> Hash for HashMap<K, V, H, A> {
     fn hash<P: Hasher>(&self, state: &mut P) {
         for i in self {
@@ -574,13 +635,67 @@ impl<K: Eq + Hash, V, H: BuildHasher, A: Allocator> Extend<(K, V)> for HashMap<K
     }
 }
 
+/// An [`Iterator`] over the keys and mutable values of a [`HashMap`].
+pub struct IterMut<'a, K, V> {
+    buckets: *mut HashMapSlot<K, V>,
+    buckets_end: *mut HashMapSlot<K, V>,
+    bucket_position: usize,
+    phantom: PhantomData<&'a mut [HashMapSlot<K, V>]>,
+}
+
+impl<'a, K, V> IterMut<'a, K, V> {
+    fn next_item(&mut self) -> Option<&'a mut Pair<K, V>> {
+        if self.buckets == self.buckets_end {
+            None
+        } else {
+            let mut ecount = unsafe { (*self.buckets).ecount };
+            let mut entries =
+                unsafe { core::ptr::addr_of_mut!((*self.buckets).entries) }.cast::<Pair<K, V>>();
+
+            while self.bucket_position >= ecount {
+                self.bucket_position = 0;
+                self.buckets = unsafe { self.buckets.offset(1) };
+                if self.buckets == self.buckets_end {
+                    return None;
+                }
+                ecount = unsafe { (*self.buckets).ecount };
+                entries = unsafe { core::ptr::addr_of_mut!((*self.buckets).entries) }
+                    .cast::<Pair<K, V>>();
+            }
+
+            let ret = unsafe { entries.add(self.bucket_position) };
+            self.bucket_position += 1;
+
+            Some(unsafe { &mut *ret })
+        }
+    }
+}
+
+impl<'a, K, V> Iterator for IterMut<'a, K, V> {
+    type Item = Pair<&'a K, &'a mut V>;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        self.next_item().map(|Pair(k, v)| Pair(k as &K, v))
+    }
+}
+
+impl<'a, K, V> FusedIterator for IterMut<'a, K, V> {}
+
+/// An [`Iterator`] over the mutable values in a [`HashMap`]
+pub struct ValuesMut<'a, K, V>(IterMut<'a, K, V>);
+
+impl<'a, K, V> Iterator for ValuesMut<'a, K, V> {
+    type Item = &'a mut V;
+
+    fn next(&mut self) -> Option<&'a mut V> {
+        self.0.next().map(|Pair(_, v)| v)
+    }
+}
+
 ///
 /// A [`HashSet`] that has a stable ABI and obtains storage backed by an [`Allocator`]
-#[allow(clippy::derive_hash_xor_eq)]
 #[derive(
-    Clone,
-    Debug, /* may not be good to derive(Debug) here. Maybe should provide impl instead */
-    Hash,
+    Clone, Debug /* may not be good to derive(Debug) here. Maybe should provide impl instead */,
 )]
 pub struct HashSet<K, H: BuildHasher = BuildHasherDefault<XLangHasher>, A: Allocator = XLangAlloc> {
     inner: HashMap<K, (), H, A>,
@@ -594,6 +709,12 @@ impl<K: Eq + Hash, H: BuildHasher, A: Allocator> PartialEq for HashSet<K, H, A>
 
 impl<K: Eq + Hash, H: BuildHasher, A: Allocator> Eq for HashSet<K, H, A> {}
 
+impl<K: Eq + Hash, H: BuildHasher, A: Allocator> Hash for HashSet<K, H, A> {
+    fn hash<Hash: Hasher>(&self, hasher: &mut Hash) {
+        self.inner.hash(hasher);
+    }
+}
+
 impl<K, H: BuildHasher + Default, A: Allocator + Default> HashSet<K, H, A> {
     /// Constructs a new [`HashSet`] with a [`Default`] allocator and [`Default`] hasher
     #[must_use]
@@ -666,6 +787,14 @@ impl<K: Eq + Hash, H: BuildHasher, A: Allocator> HashSet<K, H, A> {
             Ok(())
         }
     }
+
+    /// Removes a value from the set and returns it if present
+    pub fn remove<Q: ?Sized + Hash + Eq>(&mut self, val: &Q) -> Option<K>
+    where
+        K: Borrow<Q>,
+    {
+        self.inner.remove(val).map(|Pair(k, _)| k)
+    }
 }
 impl<K: Eq + Hash, H: BuildHasher + Default, A: Allocator + Default> FromIterator<K>
     for HashSet<K, H, A>
@@ -714,3 +843,63 @@ impl<'a, K> Iterator for SetIter<'a, K> {
         self.0.next()
     }
 }
+
+#[cfg(test)]
+mod test {
+    use super::*;
+    #[test]
+    fn test_hash_map_get() {
+        let mut map = HashMap::<&str, i32>::new();
+        map.insert("Hi", 0);
+        assert_eq!(map.get("Hi"), Some(&0));
+    }
+    #[test]
+    fn test_hash_map_missing() {
+        let mut map = HashMap::<&str, i32>::new();
+        map.insert("Hi", 0);
+        assert_eq!(map.get("Bye"), None);
+    }
+
+    #[test]
+    fn test_hash_map_iter_count() {
+        let mut map = HashMap::<&str, i32>::new();
+        map.insert("Hi", 0);
+        map.insert("Hello", 1);
+        assert_eq!(map.iter().count(), 2);
+    }
+
+    #[test]
+    fn test_hash_map_iter_mut_count() {
+        let mut map = HashMap::<&str, i32>::new();
+        map.insert("Hi", 0);
+        map.insert("Hello", 1);
+        assert_eq!(map.iter_mut().count(), 2);
+    }
+
+    #[test]
+    fn test_hash_map_iter_mut_count1() {
+        let mut map = HashMap::<&str, i32>::new();
+        map.insert("Hi", 0);
+        assert_eq!(map.iter_mut().count(), 1);
+    }
+
+    #[test]
+    fn test_hash_map_iter_present() {
+        let mut map = HashMap::<&str, i32>::new();
+        map.insert("Hi", 0);
+        map.insert("Hello", 1);
+        let mut iter = map.iter();
+        assert!(iter.any(|Pair(s, _)| *s == "Hi"));
+        assert!(!iter.any(|Pair(s, _)| *s == "Hi"));
+    }
+
+    #[test]
+    fn test_hash_map_iter_present2() {
+        let mut map = HashMap::<&str, i32>::new();
+        map.insert("Hi", 0);
+        map.insert("Hello", 1);
+        let mut iter = map.iter();
+        assert!(iter.any(|Pair(s, _)| *s == "Hello"));
+        assert!(!iter.any(|Pair(s, _)| *s == "Hello"));
+    }
+}
